@@ -1,7 +1,11 @@
+## no critic: Modules::ProhibitAutomaticExportation
+
 package Data::Dmp;
 
-our $DATE = '2017-01-30'; # DATE
-our $VERSION = '0.23'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2022-08-28'; # DATE
+our $DIST = 'Data-Dmp'; # DIST
+our $VERSION = '0.242'; # VERSION
 
 use 5.010001;
 use strict;
@@ -12,12 +16,14 @@ use Scalar::Util qw(looks_like_number blessed reftype refaddr);
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(dd dmp);
+our @EXPORT_OK = qw(dd_ellipsis dmp_ellipsis);
 
 # for when dealing with circular refs
 our %_seen_refaddrs;
 our %_subscripts;
 our @_fixups;
 
+our $OPT_MAX_DUMP_LEN_BEFORE_ELLIPSIS = 70;
 our $OPT_PERL_VERSION = "5.010";
 our $OPT_REMOVE_PRAGMAS = 0;
 our $OPT_DEPARSE = 1;
@@ -54,6 +60,14 @@ sub _double_quote {
 }
 # END COPY PASTE FROM Data::Dump
 
+# BEGIN COPY PASTE FROM String::PerlQuote
+sub _single_quote {
+    local($_) = $_[0];
+    s/([\\'])/\\$1/g;
+    return qq('$_');
+}
+# END COPY PASTE FROM String::PerlQuote
+
 sub _dump_code {
     my $code = shift;
 
@@ -113,9 +127,10 @@ sub _dump {
     my $refaddr = refaddr($val);
     $_subscripts{$refaddr} //= $subscript;
     if ($_seen_refaddrs{$refaddr}++) {
-        push @_fixups, "\$a->$subscript=\$a",
-            ($_subscripts{$refaddr} ? "->$_subscripts{$refaddr}" : ""), ";";
-        return "'fix'";
+        my $target = "\$var" .
+            ($_subscripts{$refaddr} ? "->$_subscripts{$refaddr}" : "");
+        push @_fixups, "\$var->$subscript=$target;";
+        return _single_quote($target);
     }
 
     my $class;
@@ -152,7 +167,11 @@ sub _dump {
         }
         $res .= "}";
     } elsif ($ref eq 'SCALAR') {
-        $res = "\\"._dump($$val, $subscript);
+        if (defined $class) {
+            $res = "do{my\$o="._dump($$val, $subscript).";\\\$o}";
+        } else {
+            $res = "\\"._dump($$val, $subscript);
+        }
     } elsif ($ref eq 'REF') {
         $res = "\\"._dump($$val, $subscript);
     } elsif ($ref eq 'CODE') {
@@ -166,6 +185,7 @@ sub _dump {
 }
 
 our $_is_dd;
+our $_is_ellipsis;
 sub _dd_or_dmp {
     local %_seen_refaddrs;
     local %_subscripts;
@@ -178,7 +198,12 @@ sub _dd_or_dmp {
         $res = _dump($_[0], '');
     }
     if (@_fixups) {
-        $res = "do{my\$a=$res;" . join("", @_fixups) . "\$a}";
+        $res = "do{my\$var=$res;" . join("", @_fixups) . "\$var}";
+    }
+
+    if ($_is_ellipsis) {
+        $res = substr($res, 0, $OPT_MAX_DUMP_LEN_BEFORE_ELLIPSIS) . '...'
+            if length($res) > $OPT_MAX_DUMP_LEN_BEFORE_ELLIPSIS;
     }
 
     if ($_is_dd) {
@@ -189,9 +214,12 @@ sub _dd_or_dmp {
     }
 }
 
-sub dd { local $_is_dd=1; _dd_or_dmp(@_) } # goto &sub doesn't work here
+sub dd { local $_is_dd=1; _dd_or_dmp(@_) } # goto &sub doesn't work with local
 sub dmp { goto &_dd_or_dmp }
 
+sub dd_ellipsis { local $_is_dd=1; local $_is_ellipsis=1; _dd_or_dmp(@_) }
+sub dmp_ellipsis { local $_is_ellipsis=1; _dd_or_dmp(@_) }
+
 1;
 # ABSTRACT: Dump Perl data structures as Perl code
 
@@ -207,17 +235,23 @@ Data::Dmp - Dump Perl data structures as Perl code
 
 =head1 VERSION
 
-This document describes version 0.23 of Data::Dmp (from Perl distribution Data-Dmp), released on 2017-01-30.
+This document describes version 0.242 of Data::Dmp (from Perl distribution Data-Dmp), released on 2022-08-28.
 
 =head1 SYNOPSIS
 
  use Data::Dmp; # exports dd() and dmp()
  dd [1, 2, 3]; # prints "[1,2,3]"
- $a = dmp({a => 1}); # -> "{a=>1}"
+ $var = dmp({a => 1}); # -> "{a=>1}"
+
+Print truncated dump (capped at L</$Data::Dmp::OPT_MAX_DUMP_LEN_BEFORE_ELLIPSIS>
+characters):
+
+ use Data::Dmp qw(dd_ellipsis dmp_ellipsis);
+ dd_ellipsis [1..100];
 
 =head1 DESCRIPTION
 
-Data::Dmp is a Perl dumper like L<Data::Dumper>. It's compact (only about 175
+Data::Dmp is a Perl dumper like L<Data::Dumper>. It's compact (only about 200
 lines of code long), starts fast and does not use any non-core modules except
 L<Regexp::Stringify> when dumping regexes. It produces compact single-line
 output (similar to L<Data::Dumper::Concise>). It roughly has the same speed as
@@ -228,9 +262,11 @@ on L<Data::Dump>: I removed all the parts that I don't need, particularly the
 pretty formatting stuffs) and added some features that I need like proper regex
 dumping and coderef deparsing.
 
-=head1 SETTINGS
+=head1 VARIABLES
 
-=head2 $Data::Dmp::OPT_PERL_VERSION => str (default: 5.010)
+=head2 $Data::Dmp::OPT_PERL_VERSION
+
+String, default: 5.010.
 
 Set target Perl version. If you set this to, say C<5.010>, then the dumped code
 will keep compatibility with Perl 5.10.0. This is used in the following ways:
@@ -251,7 +287,9 @@ so we replace it with:
 
 =back
 
-=head2 $Data::Dmp::OPT_REMOVE_PRAGMAS => bool (default: 0)
+=head2 $Data::Dmp::OPT_REMOVE_PRAGMAS
+
+Bool, default: 0.
 
 If set to 1, then pragmas at the start of coderef dump will be removed. Coderef
 dump is produced by L<B::Deparse> and is of the form like:
@@ -265,50 +303,88 @@ turn turn on this option which will make the above dump become:
 
 Note that without the pragmas, the dump might be incorrect.
 
-=head2 $Data::Dmp::OPT_DEPARSE => bool (default: 1)
+=head2 $Data::Dmp::OPT_DEPARSE
+
+Bool, default: 1.
 
 Can be set to 0 to skip deparsing code. Coderefs will be dumped as
 C<sub{"DUMMY"}> instead, like in Data::Dump.
 
-=head2 $Data::Dmp::OPT_STRINGIFY_NUMBERS => bool (default: 0)
+=head2 $Data::Dmp::OPT_STRINGIFY_NUMBERS
+
+Bool, default: 0.
 
 If set to true, will dump numbers as quoted string, e.g. 123 as "123" instead of
 123. This might be helpful if you want to compute the hash of or get a canonical
 representation of data structure.
 
+=head2 $Data::Dmp::OPT_MAX_DUMP_LEN_BEFORE_ELLIPSIS
+
+Int, default: 70.
+
+Used by L</dd_ellipsis> and L</dmp_ellipsis>.
+
 =head1 BENCHMARKS
 
  [1..10]:
-                       Rate    Data::Dump Data::Dumper Data::Dmp
- Data::Dump     30417+-55/s            --       -66.2%    -74.0%
- Data::Dumper   89888+-79/s  195.52+-0.6%           --    -23.1%
- Data::Dmp    116890+-160/s 284.29+-0.87% 30.04+-0.21%        --
+              Rate/s Precision/s  Data::Dump Data::Dumper Data::Dmp
+ Data::Dump    24404          95          --       -61.6%    -75.6%
+ Data::Dumper  63580         210 160.5+-1.3%           --    -36.4%
+ Data::Dmp     99940         130 309.5+-1.7% 57.18+-0.55%        --
  
  [1..100]:
-                        Rate    Data::Dump  Data::Dmp Data::Dumper
- Data::Dump    3712.3+-7.9/s            --     -73.9%       -74.9%
- Data::Dmp    14211.3+-4.9/s 282.82+-0.82%         --        -3.8%
- Data::Dumper    14771+-28/s   297.9+-1.1% 3.94+-0.2%           --
+               Rate/s Precision/s  Data::Dump Data::Dumper Data::Dmp
+ Data::Dump    2934.3         7.8          --       -75.3%    -76.2%
+ Data::Dumper   11873          32 304.6+-1.5%           --     -3.7%
+ Data::Dmp    12323.4           4   320+-1.1%   3.8+-0.28%        --
  
  Some mixed structure:
-                     Rate    Data::Dump    Data::Dmp Data::Dumper
- Data::Dump    8764+-16/s            --       -67.6%       -80.1%
- Data::Dmp    27016+-36/s  208.28+-0.7%           --       -38.6%
- Data::Dumper 43995+-13/s 402.02+-0.95% 62.85+-0.22%           --
+              Rate/s Precision/s   Data::Dump   Data::Dmp Data::Dumper
+ Data::Dump     7161          12           --      -69.3%       -78.7%
+ Data::Dmp     23303          29 225.43+-0.7%          --       -30.6%
+ Data::Dumper  33573          56  368.8+-1.1% 44.07+-0.3%           --
 
 =head1 FUNCTIONS
 
-=head2 dd($data, ...) => $data ...
+=head2 dd
+
+Usage:
+
+ dd($data, ...); # returns $data
 
 Exported by default. Like C<Data::Dump>'s C<dd> (a.k.a. C<dump>), print one or
 more data to STDOUT. Unlike C<Data::Dump>'s C<dd>, it I<always> prints and
 return I<the original data> (like L<XXX>), making it convenient to insert into
 expressions. This also removes ambiguity and saves one C<wantarray()> call.
 
-=head2 dmp($data, ...) => $str
+=head2 dmp
+
+Usage:
+
+ my $dump = dmp($data, ...);
 
 Exported by default. Return dump result as string. Unlike C<Data::Dump>'s C<dd>
-(a.k.a. C<dump>), it I<never> prints and only return the data.
+(a.k.a. C<dump>), it I<never> prints and only return the dump result.
+
+=head2 dd_ellipsis
+
+Usage:
+
+ dd_ellipsis($data, ...); # returns data
+
+Just like L</dd>, except will truncate its output to
+L</$Data::Dmp::OPT_MAX_DUMP_LEN_BEFORE_ELLIPSIS> characters if dump is too long.
+Note that truncated dump will probably not be valid Perl code.
+
+=head2 dmp_ellipsis
+
+Usage:
+
+ my $dump = dd_ellipsis($data, ...); # returns data
+
+Just like L</dmp>, except will truncate dump result to
+L</$Data::Dmp::OPT_MAX_DUMP_LEN_BEFORE_ELLIPSIS> characters if dump is too long.
+Note that truncated dump will probably not be valid Perl code.
 
 =head1 FAQ
 
@@ -354,14 +430,6 @@ Please visit the project's homepage at L<https://metacpan.org/release/Data-Dmp>.
 
 Source repository is at L<https://github.com/perlancar/perl-Data-Dmp>.
 
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Data-Dmp>
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
 =head1 SEE ALSO
 
 L<Data::Dump> and other variations/derivate works in Data::Dump::*.
@@ -376,11 +444,37 @@ L<YAML>, L<JSON>, L<Storable>, L<Sereal>, and other serialization formats.
 
 perlancar <perlancar@cpan.org>
 
+=head1 CONTRIBUTING
+
+
+To contribute, you can send patches by email/via RT, or send pull requests on
+GitHub.
+
+Most of the time, you don't need to build the distribution yourself. You can
+simply modify the code, then test via:
+
+ % prove -l
+
+If you want to build the distribution (e.g. to try to install it locally on your
+system), you can install L<Dist::Zilla>,
+L<Dist::Zilla::PluginBundle::Author::PERLANCAR>,
+L<Pod::Weaver::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
+Dist::Zilla- and/or Pod::Weaver plugins. Any additional steps required beyond
+that are considered a bug and can be reported to me.
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017 by perlancar@cpan.org.
+This software is copyright (c) 2022, 2021, 2020, 2017, 2016, 2015, 2014 by perlancar <perlancar@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Data-Dmp>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =cut
